<html>
	<head>
		<meta http-equiv="Content-Type" content="text/html charset=utf-8">
	</head>
	<body style="word-wrap: break-word; -webkit-nbsp-mode: space; -webkit-line-break: after-white-space;" class="">
		<p>
		Dear $fullname,
		</p>
		<p>
		your $object is going to expire within $days_left days on $expiry_date.
		</p>
		<p>
		Best regards,
		&nsbp;Your Admin Team
		</p>
	</body>
</html>
